#!/usr/bin/python
# -*- coding:utf-8 -*-

"""An abstract class of scoreboard classes.
"""

from dom import *
import re
import scorecard
import makelogo

def newInstance():
    """This method must be inplemented in subclasses which instanciate
    an object."""
    return NPB_NS_2005()


class NPB_NS_2005( scorecard.ScoreCard ):
    def __init__( self ):
        scorecard.ScoreCard.__init__( self )
        self.games = []
        return

    """Overridable methods"""
    
    def getSportsCategory( self ):
        return "baseball"

    def getCountry( self ):
        return "Japan"

    def getName( self ):
        return "プロ野球 (nikkansports.com)"

    def getVersion( self ):
        return "1.1"

    def getIcon( self, team ):
        if team == "中日":          name = "d"
        elif team == "ヤクルト":     name = "s"
        elif team == "巨人":        name = "g"
        elif team == "阪神":        name = "t"
        elif team == "広島":        name = "c"
        elif team == "横浜":        name = "yb"
        elif team == "西武":        name = "l"
        elif team == "ソフトバンク": name = "h"
        elif team == "日本ハム":     name = "f"
        elif team == "ロッテ":      name = "m"
        elif team == "オリックス":   name = "bs"
        elif team == "楽天":        name = "e"
        try:
            uri = makelogo.getURI( 'http://www.npb.or.jp/images/npb/pet40inter_%s.gif' % name, 'npb', name, transparency=None )
            return '<img src="%s" class="logo"/>' % uri
        except Exception, e:
            return ''
        pass

    def getEncoding( self ):
        return 'sjis'

    def getLanguage( self ):
        return "ja"

    def applyFilter( self, contents ):
        return re.sub( "<tr[^>]+>", "<tr>", contents )
    
    def getURL( self ):
        return "http://www.nikkansports.com/ns/baseball/professional/score/pf-nscore.html"

    def __normalize( self, text ):
        text = unicode( text, 'utf-8' )
        text = re.sub( u'[　\\s【】\[\]]+', '', text )
        t = ""
        for c in text:
            try:
                n = u'０１２３４５６７８９'.index( c )
                t += "%d" % n
            except:
                t += c
                pass
            pass
        return t.encode( 'utf-8' )

    def interpreteDocument( self, document ):
        import sys, re
        elem_info = document.get_element_at_path( "/html/table/tr/td/table/tr" )
        self.last_update = elem_info.get_text( ',' ).split( ',' )[ -1 ]
        elem_game = document.get_element_at_path( "/html/table,2/tr/td/table" )

        elem_game = document.get_element_at_path( "/html/table,2/tr/td" ).get_children()[ 0 ]

        children = document.get_element_at_path( "/html/table,2/tr/td" ).get_children()

        games = []
        game = None
        for child in children[ 2: ]:
            if isinstance( child, Element ):
                if child.get_element_at_path( 'tr,3' ) != None:
                    omote = self.__normalize( child.get_element_at_path( "tr,2" ).get_text( ',' ) )
                    ura = self.__normalize( child.get_element_at_path( "tr,3" ).get_text( ',' ) )
                    omote = omote.strip( ',' ).split( ',' )
                    ura   = ura.strip( ',' ).split( ',' )
                    num   = len( omote ) - 2
                    if game != None: 
                        game[ 'omote' ] = omote
                        game[ 'kai' ] = num
                        game[ 'ura' ] = ura
                        pass
                    pass
                else:
                    text = child.get_text( ' ' ) 
                    items = re.split( '◇', text )
                    if len( items ) > 1:
                        if game != None: games.append( game )
                        place = items[ -1 ].strip()
                        starttime = items[ -2 ].strip()
                        status = self.__normalize( items[ 0 ].strip() )
                        game = { 'place':place, 'status':status, 'time':starttime,
                                 'kai':9, 'omote':None, 'ura':None, 'info':'' }
                        pass
                    elif game != None:
                        if game[ 'omote' ] != None and game[ 'ura' ] != None:
                            text = self.__normalize( text )
                            if text.count( game[ 'omote' ][ 0 ] ) > 0 and text.count( game[ 'ura' ][ 0 ] ) > 0:
                                text = re.sub( game[ 'omote' ][ 0 ], '<br/><span class="team">' + game[ 'omote' ][ 0 ] + "</span>&nbsp;", text  )
                                text = re.sub( game[ 'ura' ][ 0 ], '<br/><span class="team">' + game[ 'ura' ][ 0 ] + "</span>&nbsp;", text  )
                                text = re.sub( '本塁打', '<br/>本塁打&nbsp;', text )
                                game[ 'info' ] = text
                                pass
                            pass
                        pass
                    pass
                pass
            pass
        self.games = games
        return

    def getStartKeyword( self ):
        return "<!--====コンテンツエリア===-->"

    def getEndKeyword( self ):
        return "<!--====コンテンツエリア===-->"

    def toHTML( self ):
        html  = self.html_header()
        if len( self.games ) == 0:
            html += "<span class=\"tabletitle\">今日のゲームはありません</span>"
            return html + self.html_footer()
        name = re.sub( '\([^\)]*\)\\s*$', '', self.getName() )
        html += '<span class="tabletitle">%s</span><br/>' % name
        for game in self.games:
            html += '<span class="category">%s %s</span><br/>\n' % ( self.__normalize( game[ 'place' ] ), game[ 'status' ] )
            html += '<table>'
            omote = game[ 'omote' ]
            ura = game[ 'ura' ]
            html += '<tr><td class="team"></td>'
            turns = len( omote ) - 2
            for i in range( turns ):
                html += '<td class="pt">%d</td>' % ( i + 1 )
                pass
            
            html += '<td class="pt"></td></tr>'
            for row in ( omote, ura ):
                html += '<tr><td class="team">%s%s</td>' % ( self.getIcon( row[ 0 ] ), row[ 0 ] )
                for i in range( 1, len( row ) - 1 ):
                    html += '<td class="pt">%s</td>' % row[ i ]
                    pass
                if len( row ) > 2:
                    html += '<td class="total">%s</td>' % row[ -1 ]
                    pass
                html += '</tr>\n'
            html += '</table>'
            if game[ 'status' ].count( '試合開始前' ) == 0:
                html += '<div class="info">%s</div>' % game[ 'info' ]
                pass
            pass
        html += self.html_footer()
        return html

    def html_css_plugin( self ):
        return """<style type=\"text/css\">
        .pt { width:14px; height:10px; }
        .total { width:14px; height:10px; font-weight:bold;}
        .team { height:10px; text-align:left; width:80px;}
        .logo { width:10px;height:10px;}
        .info { font-family:\"HiraMaru-Pro\" \"Lucida Grande\" serif; font-size:10px; margin:2px 20px 20px 2px;}
        
        </style>
        """

    pass

